@@ -1,5 +1,6 @@
 #include "mapper.h"
 #include "dynamic.h"
+#include "servicedef.h"
 
 #undef New
 
@@ -734,6 +735,10 @@ const char *Mapper::full_name() const {
     return message_def->full_name();
 }
 
+const char *Mapper::package_name() const {
+    return HvNAME(stash);
+}
+
 MapperField *Mapper::find_extension(const std::string &name) const {
     for (vector<MapperField *>::const_iterator it = extension_mapper_fields.begin(), en = extension_mapper_fields.end(); it != en; ++it) {
         if (name == (*it)->name())
@@ -2047,6 +2052,98 @@ SV *EnumMapper::enum_descriptor() const {
     return ref;
 }
 
+ServiceMapper::ServiceMapper(pTHX_ Dynamic *_registry, const gpd::ServiceDef *_service_def) :
+        registry(_registry),
+        service_def(_service_def) {
+    SET_THX_MEMBER;
+
+    registry->ref();
+}
+
+ServiceMapper::~ServiceMapper() {
+    delete service_def;
+    // make sure this only goes away after inner destructors have completed
+    refcounted_mortalize(aTHX_ registry);
+}
+
+SV *ServiceMapper::service_descriptor() const {
+    SV *ref = newSV(0);
+
+    sv_setref_iv(ref, "Google::ProtocolBuffers::Dynamic::ServiceDef", (IV) service_def);
+
+    return ref;
+}
+
+MethodMapper::MethodMapper(pTHX_ Dynamic *_registry, const string &method, const MessageDef *_input_def, const MessageDef *_output_def, bool client_streaming, bool server_streaming) :
+        registry(_registry),
+        input_def(_input_def),
+        output_def(_output_def) {
+    SET_THX_MEMBER;
+
+    registry->ref();
+
+    method_name_key_sv = newSVpvs_share("method");
+    serialize_key_sv = newSVpvs_share("serialize");
+    deserialize_key_sv = newSVpvs_share("deserialize");
+    method_name_sv = newSVpvn_share(method.data(), method.length(), 0);
+    serialize_sv = NULL;
+    deserialize_sv = NULL;
+
+    const char *grpc_name;
+    if (client_streaming) {
+        if (server_streaming) {
+            grpc_name = "Grpc::Client::BaseStub::_bidiRequest";
+        } else {
+            grpc_name = "Grpc::Client::BaseStub::_clientStreamRequest";
+        }
+    } else {
+        if (server_streaming) {
+            grpc_name = "Grpc::Client::BaseStub::_serverStreamRequest";
+        } else {
+            grpc_name = "Grpc::Client::BaseStub::_simpleRequest";
+        }
+    }
+
+    grpc_call_sv = get_cv(grpc_name, 0);
+    if (grpc_call_sv == NULL)
+        croak("Unable to resolve function '%s'", grpc_name);
+}
+
+MethodMapper::~MethodMapper() {
+    SvREFCNT_dec(method_name_key_sv);
+    SvREFCNT_dec(serialize_key_sv);
+    SvREFCNT_dec(deserialize_key_sv);
+    SvREFCNT_dec(method_name_sv);
+    SvREFCNT_dec(serialize_sv);
+    SvREFCNT_dec(deserialize_sv);
+    SvREFCNT_dec(grpc_call_sv);
+    // make sure this only goes away after inner destructors have completed
+    refcounted_mortalize(aTHX_ registry);
+}
+
+void MethodMapper::resolve_input_output() {
+    const Mapper *request_mapper = registry->find_mapper(input_def);
+    const Mapper *response_mapper = registry->find_mapper(output_def);
+
+    string request_encoder_name =
+        request_mapper->package_name()
+        + string("::_static_encode");
+    SV *encode = (SV *) get_cv(request_encoder_name.c_str(), 0);
+    if (encode == NULL)
+        croak("Unable to find GRPC encoder in package '%s' for message '%s'",
+              request_mapper->package_name(), request_mapper->full_name());
+    serialize_sv = newRV_inc(encode);
+
+    string response_decoder_name =
+        response_mapper->package_name()
+        + string("::_static_decode");
+    SV *decode = (SV *) get_cv(response_decoder_name.c_str(), 0);
+    if (decode == NULL)
+        croak("Unable to find GRPC decoder in package '%s' for message '%s'",
+              response_mapper->package_name(), response_mapper->full_name());
+    deserialize_sv = newRV_inc(decode);
+}
+
 WarnContext::WarnContext(pTHX) : chained_handler(NULL) {
     CV *handler = get_cv("Google::ProtocolBuffers::Dynamic::Mapper::handle_warning", 0);
 
